/*
    rtnet/module/rt_echo_client/rt_echo_client.c
    module that tests echo capability of server

    rtnet - real-time networking subsystem
    Copyright (C) 1999,2000 Zentropic Computing, LLC

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#define FREQ            1                   // Basic frequency in Hz
#define NTASKS          1

#define TICK_PERIOD	1000000

#include <linux/module.h>

#include <rtnet/rtnet.h>

#include <rtai/rtai.h>
#include <rtai/rtai_sched.h>

void *process(void * arg);

RT_TASK rt_task;

/* hard coded IP address 192.168.1.1 */
#define SERV_ADDR 0x0101a8c0

#define SERV_PORT 7

static int sock;
static struct sockaddr_in servaddr;

unsigned long tsc1,tsc2;

void *process(void * arg)
{
	char *sendline="this is a test";
	int ret;

	rt_printk("task started\n");
	while(1){
		rt_task_wait_period();

		rdtscl(tsc1);
		ret=rt_sendto(sock,sendline,strlen(sendline),0,
			(struct sockaddr *)&servaddr,sizeof(servaddr));
		rt_printk("sendto()=%d\n",ret);
	}
}

int echo_rcv(int s,void *arg)
{
	rdtscl(tsc2);
	rt_printk("rcv diff=%ld\n",tsc2,tsc2-tsc1);

	return 0;
}

int init_module(void)
{
	int ret;
	RTIME tick_period;

	// set up sending
	
	sock=rt_socket(AF_INET,SOCK_DGRAM,0);

	memset(&servaddr,0,sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = 0;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);

	ret=rt_bind(sock,(struct sockaddr *)&servaddr,sizeof(servaddr));
	printk("rt_bind=%d\n",ret);

	memset(&servaddr,0,sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(SERV_PORT);
	servaddr.sin_addr.s_addr = SERV_ADDR;

	// set up receiving

	rt_socket_callback(sock,echo_rcv,NULL);

	// set up task

	rt_printk("starting task\n");

	tick_period = start_rt_timer(nano2count(TICK_PERIOD));

	ret=rt_task_init(&rt_task,(void *)process,0,4096,0,0,NULL);
	printk("rt_task_init returned %d\n",ret);

	ret=rt_task_make_periodic_relative_ns( &rt_task, 1000000, 1000000000);
	printk("rt_task_make_periodic returned %d\n",ret);

	return 0;
}


void cleanup_module(void)
{
	stop_rt_timer();
	rt_task_delete(&rt_task);

	rt_close(sock);
}


