/*
    linux/net/rt/echo_server.c
    RT networking UDP echo server (port 7)

    Linux RT networking
    Copyright (C) 1999,2000 Zentropic Computing, LLC

    Author:	David Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/socket.h>
#include <linux/errno.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/init.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/skbuff.h>

#include <rtnet/rtnet.h>

#define ECHO_PORT 7

#define DEBUG 0

//static struct rtsocket *echo_sock;
static int echo_sock;

#define BUFSZ 100

char buffer[BUFSZ];

static int echo_rcv(int s,void *arg)
{
	int ret;
	struct msghdr msg;
	struct iovec iov;
	struct sockaddr_in addr;

	memset(&msg,0,sizeof(msg));
	iov.iov_base=buffer;
	iov.iov_len=BUFSZ;
	msg.msg_name=NULL;
	msg.msg_iov=&iov;
	msg.msg_iovlen=1;
	msg.msg_control=NULL;
	msg.msg_controllen=0;
	msg.msg_name=&addr;
	msg.msg_namelen=sizeof(addr);

	ret=rt_recvmsg(echo_sock,&msg,0);

#if DEBUG
	rt_printk("recvmsg()=%d\n",ret);
#endif

	if(ret>0){
		//rt_printk("echo rcv from %08x.%d\n",(int)ntohl(addr.sin_addr.s_addr),(int)ntohs(addr.sin_port));

		rt_sendmsg(echo_sock,&msg,0);
#if DEBUG
		rt_printk("sendmsg()=%d\n",ret);
#endif
	}

	return 0;
}


int init_module(void)
{
	struct sockaddr_in addr;

	echo_sock = rt_socket(AF_INET,SOCK_DGRAM,0);

	memset(&addr,0,sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(ECHO_PORT);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);

	rt_bind(echo_sock,(struct sockaddr *)&addr,sizeof(addr));

	rt_socket_callback(echo_sock,echo_rcv,NULL);

	printk("rt_echo_server started\n");

	return 0;
}

void cleanup_module(void)
{
	printk("rt_echo_server stopped\n");

	rt_close(echo_sock);
}



