/*
 * INET		An implementation of the TCP/IP protocol suite for the LINUX
 *		operating system.  INET is implemented using the  BSD Socket
 *		interface as the means of communication with the user level.
 *
 *		INET protocol dispatch tables.
 *
 * Version:	$Id: protocol.c,v 1.2 2000/07/05 16:23:31 ds Exp $
 *
 * Authors:	Ross Biro, <bir7@leland.Stanford.Edu>
 *		Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *
 * Fixes:
 *		Alan Cox	: Ahah! udp icmp errors don't work because
 *				  udp_err is never called!
 *		Alan Cox	: Added new fields for init and ready for
 *				  proper fragmentation (_NO_ 4K limits!)
 *		Richard Colella	: Hang on hash collision
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/config.h>
#include <linux/socket.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/timer.h>
#include <net/ip.h>
#include <net/protocol.h>
#include <net/tcp.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <net/icmp.h>
#include <net/udp.h>
#include <net/ipip.h>
#include <linux/igmp.h>
#include <rtnet/rtnet.h>


/*
 *	Find a protocol in the protocol tables given its
 *	IP type.
 */

struct rtsocket *rt_inet_get_protocol(unsigned char prot)
{
	struct rtsocket *p;

	for(p=rt_ip_sockets;p;p=p->next){
		if(p->proto == prot)
			return p;
	}
	return NULL;
}

/*
 *	Add a protocol handler to the hash tables
 */

void rt_inet_add_protocol(struct rtsocket *prot)
{
	prot->next=rt_ip_sockets;
	rt_ip_sockets=prot;
}

/*
 *	Remove a protocol from the hash tables.
 */
 
int rt_inet_del_protocol(struct rtsocket *prot)
{
	printk("rt_inet_del_protocol() not implemented\n");
	return(-1);
}

