/*
 *  rt_printk.c, hacked from linux/kernel/printk.c
 *
 * Modified for RT support, David Schleef
 */

#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/sched.h>
#include <asm/irq.h>
#include <asm/ptrace.h>
#if 0
#include <linux/rtl.h>
#endif
#include <rtnet/rtnet.h>
#include <linux/string.h>

#undef USE_IRQ
#define USE_TIMER

#define BUF_LEN	(16384)

static char rt_printk_buf[BUF_LEN];
static char buf[1024];

static int buf_front,buf_back;

#ifdef USE_TIMER
static struct timer_list rt_printk_timer;
#endif

#ifdef USE_IRQ
static int rt_printk_irq;
static void rt_printk_interrupt(int irq,void *junk,struct pt_regs *regs);
#endif

/* this is rather bogus, but it will catch most errors */
static volatile int rt_printk_lock;


int __rt_printk(const char *fmt, ...)
{
	va_list args;
	int len,i;

#if 0
	if(rtl_rt_system_is_idle()){
		va_start(args, fmt);
		len=printk(fmt,args);
		va_end(args);
		return len;
	}
#endif

#if 0
	if(rt_printk_lock){
		/* force a panic */
		*(int *)0=0;
	}
#endif
	rt_printk_lock=1;

	va_start(args, fmt);
	len = vsprintf(buf, fmt, args); /* hopefully i < sizeof(buf)-1 */
	va_end(args);

	if(buf_front+len>=BUF_LEN){
		i=BUF_LEN-buf_front;
		memcpy(rt_printk_buf+buf_front,buf,i);
		memcpy(rt_printk_buf,buf+i,len-i);
		buf_front=len-i;
	}else{
		memcpy(rt_printk_buf+buf_front,buf,len);
		buf_front+=len;
	}

	rt_printk_lock=0;

#ifdef USE_IRQ
	rtl_global_pend_irq(rt_printk_irq);
#endif

	return len;
}


#ifdef USE_IRQ
void rt_printk_interrupt(int irq,void *junk,struct pt_regs *regs)
{
	int tmp;

	for(;;){
		tmp=buf_front;
		if(buf_back>tmp){
			printk("%.*s",BUF_LEN-buf_back,rt_printk_buf+buf_back);
			buf_back=0;
		}
		if(buf_back==tmp)break;
		printk("%.*s",tmp-buf_back,rt_printk_buf+buf_back);
		buf_back=tmp;
	}
}
#endif

#ifdef USE_TIMER
void rt_printk_timeout(unsigned long x)
{
	int tmp;

	for(;;){
		tmp=buf_front;
		if(buf_back>tmp){
			printk("%.*s",BUF_LEN-buf_back,rt_printk_buf+buf_back);
			buf_back=0;
		}
		if(buf_back==tmp)break;
		printk("%.*s",tmp-buf_back,rt_printk_buf+buf_back);
		buf_back=tmp;
	}

	rt_printk_timer.expires=jiffies+1;
	add_timer(&rt_printk_timer);
}
#endif

#define INIT_MODULE rt_printk_init
#define CLEANUP_MODULE cleanup_rt_printk

void CLEANUP_MODULE(void)
{
#ifdef USE_IRQ
	free_irq(rt_printk_irq,NULL);
	rt_printk_irq=0;
#endif

#ifdef USE_TIMER
	del_timer(&rt_printk_timer);
#endif
}


int INIT_MODULE(void)
{
#ifdef USE_TIMER
	init_timer(&rt_printk_timer);

	rt_printk_timer.function=rt_printk_timeout;
	rt_printk_timer.expires=jiffies+1;

	add_timer(&rt_printk_timer);
#endif

#if USE_IRQ
	rt_printk_irq=rtl_get_soft_irq(rt_printk_interrupt,"rt_printk");
	if(rt_printk_irq<0){
		printk("rt_printk: can't allocate soft irq\n");
		return -EIO;
	}
#endif

	return 0;
}


